"""Specify a colors for cells
and points of a Mesh"""
from vedo import dataurl, Plotter, Mesh

plt = Plotter(N=3, axes=11)

##################################### add a cell array
man1 = Mesh(dataurl+"man_low.vtk").linewidth(0.1)
nv = man1.ncells                           # nr. of cells
scals = range(nv)                          # coloring by the index of cell

man1.cmap("Paired", scals, on='cells').add_scalarbar("cell nr")
plt.at(0).show(man1, __doc__, elevation=-60)


##################################### Point coloring
man2 = Mesh(dataurl+"man_low.vtk")
scals = man2.points()[:, 0] + 37           # pick x coordinates of vertices

man2.cmap("hot", scals)
man2.add_scalarbar(horizontal=True)
plt.at(1).show(man2, "mesh.cmap()")


##################################### Cell coloring
man3 = Mesh(dataurl+"man_low.vtk")
scals = man3.cell_centers()[:, 2] + 37     # pick z coordinates of cells
man3.cmap("afmhot", scals, on='cells')

# add a fancier 3D scalar bar embedded in the scene
man3.add_scalarbar3d(size=[None,3])
man3.scalarbar.rotate_x(90).y(0.2)
plt.at(2).show(man3, "mesh.cmap(on='cells')")

plt.interactive().close()



import sys
from PyQt5 import Qt
from vtk.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor
from vedo import Plotter, Cone


class MainWindow(Qt.QMainWindow):

    def __init__(self, parent=None):

        Qt.QMainWindow.__init__(self, parent)
        self.frame = Qt.QFrame()
        self.layout = Qt.QVBoxLayout()
        self.widget = QVTKRenderWindowInteractor(self.frame)

        # Create renderer and add the vedo objects and callbacks
        self.plt = Plotter(N=2, axes=1, qt_widget=self.widget)
        self.id1 = self.plt.add_callback("mouse click", self.onMouseClick)
        self.id2 = self.plt.add_callback("key press", self.onKeypress)

        cone1 = Cone().rotate_x(20)
        cone2 = Cone().rotate_x(40).c("blue5")

        self.plt.at(0).show(cone1)
        self.plt.at(1).show(cone2)

        # Set up the rest of the Qt window
        button = Qt.QPushButton("My Button makes the cone red")
        button.setToolTip("This is an example button")
        button.clicked.connect(self.onClick)
        self.layout.addWidget(self.widget)
        self.layout.addWidget(button)
        self.frame.setLayout(self.layout)
        self.setCentralWidget(self.frame)
        self.show()  # NB: qt, not a Plotter method

    def onMouseClick(self, evt):
        print("mouse clicked")

    def onKeypress(self, evt):
        print("key pressed:", evt.keypress)

    @Qt.pyqtSlot()
    def onClick(self):
        self.plt.actors[0].color("red5").rotate_z(40)
        self.plt.render()

    def onClose(self):
        self.widget.close()


if __name__ == "__main__":

    app = Qt.QApplication(sys.argv)
    window = MainWindow()
    app.aboutToQuit.connect(window.onClose)
    app.exec_()



import sys
from PyQt5 import Qt
from vtk.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor
from vedo import Plotter, Cone, printc

class MainWindow(Qt.QMainWindow):

    def __init__(self, parent=None):

        Qt.QMainWindow.__init__(self, parent)
        self.frame = Qt.QFrame()
        self.layout = Qt.QVBoxLayout()
        self.vtkWidget = QVTKRenderWindowInteractor(self.frame)

        # Create renderer and add the vedo objects and callbacks
        self.plt = Plotter(qt_widget=self.vtkWidget)
        self.id1 = self.plt.add_callback("mouse click", self.onMouseClick)
        self.id2 = self.plt.add_callback("key press",   self.onKeypress)
        self.plt += Cone().rotate_x(20)
        self.plt.show()                  # <--- show the vedo rendering

        # Set-up the rest of the Qt window
        button = Qt.QPushButton("My Button makes the cone red")
        button.setToolTip('This is an example button')
        button.clicked.connect(self.onClick)
        self.layout.addWidget(self.vtkWidget)
        self.layout.addWidget(button)
        self.frame.setLayout(self.layout)
        self.setCentralWidget(self.frame)
        self.show()                     # <--- show the Qt Window

    def onMouseClick(self, evt):
        printc("You have clicked your mouse button. Event info:\n", evt, c='y')

    def onKeypress(self, evt):
        printc("You have pressed key:", evt.keypress, c='b')

    @Qt.pyqtSlot()
    def onClick(self):
        printc("..calling onClick")
        self.plt.actors[0].color('red').rotate_z(40)
        self.plt.interactor.Render()

    def onClose(self):
        #Disable the interactor before closing to prevent it
        #from trying to act on already deleted items
        printc("..calling onClose")
        self.vtkWidget.close()

if __name__ == "__main__":
    app = Qt.QApplication(sys.argv)
    window = MainWindow()
    app.aboutToQuit.connect(window.onClose) # <-- connect the onClose event
    app.exec_()



"""Create 2 independent timer callbacks:"""
from vedo import *

# Defining a function to be called by a timer event
def func1(event):
    # Check if this function was called by the right timer
    if event.timerid != ida:
        return
    # Rotate a cube mesh and set its color to green5
    msh.rotate_z(1.0).c("green5")
    # Update text and print a message with the event and timer ids
    txt.text("func1() called").background('green5')
    printc(f"func1() id={event.id}, timerid={event.timerid}", c='g')
    plt.render()

# Defining another function to be called by a different timer event
def func2(event):
    # Check if this function was called by the right timer
    if event.timerid != idb:
        return
    # Rotate the same cube mesh in a different direction
    msh.rotate_x(5.0).c("red5")
    # Update text and print a message with the event and timer ids
    txt.text("func2() called").background('red5')
    printc(f"func2() id={event.id}, timerid={event.timerid}", c='r')
    plt.render()

# Create a cube mesh and a text object
msh = Cube()
txt = Text2D(font="Calco", pos='top-right')

# Create a plotter object with axes
plt = Plotter(axes=1)

# Add the two callback functions to the plotter's timer events
id1 = plt.add_callback("timer", func1)
id2 = plt.add_callback("timer", func2)
printc("Creating Timer Callbacks with IDs:", id1, id2)

# Start two timers, one with a delay of 1s and the other with a delay of 2.3s
ida = plt.timer_callback("start", dt=1000)
idb = plt.timer_callback("start", dt=2300)
printc("Starting timers with IDs         :", ida, idb)

# Stop the first timer using its ID
# plt.timer_callback("stop", ida)

plt.show(msh, txt, __doc__, viewup='z')
plt.close()



"""Create a Volume
from a numpy object using imread"""
from vedo import *
from skimage.io import imread


f = dataurl+'embryo.tif'

voriginal = Volume(f)
printc('voxel size is', voriginal.spacing(), c='cyan')

raw = imread(f)
narray = np.transpose(raw, axes=[2, 1, 0])

vraw = Volume(narray, spacing=(104,104,104))

# Compare loading the volume directly with the numpy volume:
# they should be the same
show(
    [(voriginal, __doc__),
    (vraw,"From imread\n(should be same as left)")],
    N=2,
    axes=1,
).close()




"""
Example usage of cmap() to assign a color to each mesh vertex
by looking it up in matplotlib database of colormaps
"""
from vedo import Plotter, Mesh, dataurl
print(__doc__)

# these are the some matplotlib color maps
maps = [
    "afmhot",
    "binary",
    "bone",
    "cool",
    "coolwarm",
    "copper",
    "gist_earth",
    "gray",
    "hot",
    "jet",
    "rainbow",
    "winter",
]

mug = Mesh(dataurl+"mug.ply")
scalars = mug.points()[:, 1]  # let y-coord be the scalar

plt = Plotter(N=len(maps))

for i, key in enumerate(maps):  # for each available color map name
    imug = mug.clone(deep=False).cmap(key, scalars, n_colors=5)
    plt.at(i).show(imug, key)

plt.interactive().close()



"""Slice/probe a Volume with a Mesh"""
from vedo import *

vol = Volume(dataurl+'embryo.slc').mode(1).c('bone')
msh = Paraboloid(res=200).scale(200).pos(100,100,200)

scals = probe_points(vol, msh).pointdata[0]
msh.cmap('Spectral', scals).add_scalarbar()

show(vol, msh, __doc__, axes=True).close()
